@@ -28,6 +28,7 @@ use WWW::GoodData::Agent;
 use JSON;
 use URI;
 
+our $VERSION = '1.10';
 our $root = new URI ('https://secure.gooddata.com/gdc');
 
 =head1 METHODS
@@ -248,7 +249,7 @@ sub logout
 
 	# Forget Basic authentication
 	my $root = new URI ($self->{agent}{root});
-	my $staging = $self->get_uri ('uploads');
+	my $staging = $self->get_uri ('uploads')->abs ($root);
 	my $netloc = $staging->host.':'.$staging->port;
 	$self->{agent}->credentials ($netloc,
 		'GoodData project data staging area', undef, undef);
@@ -322,14 +323,17 @@ sub delete_project
 	$self->{agent}->delete ($uri);
 }
 
-=item B<create_project> TITLE SUMMARY TEMPLATE
+=item B<create_project> TITLE SUMMARY TEMPLATE DRIVER TOKEN
 
-Create a project given its title and optionally summary and project template,
+Create a project given its title and optionally summary, project template,
+db engine driver and authorization token
 return its identifier.
 
 The list of valid project templates is available from the template server:
 L<https://secure.gooddata.com/projectTemplates/>.
 
+Valid db engine drivers are 'Pg' (default) and 'mysql'.
+
 =cut
 
 sub create_project
@@ -338,6 +342,8 @@ sub create_project
 	my $title = shift or die 'No title given';
 	my $summary = shift || '';
 	my $template = shift;
+	my $driver= shift;
+	my $token = shift;
 
 	# The redirect magic does not work for POSTs and we can't really
 	# handle 401s until the API provides reason for them...
@@ -345,8 +351,12 @@ sub create_project
 
 	return $self->{agent}->post ($self->get_uri ('projects'), {
 		project => {
-			# No hook to override this; use web UI
-			content => { guidedNavigation => 1 },
+			content => {
+				# No hook to override this; use web UI
+				guidedNavigation => 1,
+				($driver ? (driver => $driver) : ()),
+				($token ? (authorizationToken => $token) : ())
+			},
 			meta => {
 				summary => $summary,
 				title => $title,
@@ -355,61 +365,47 @@ sub create_project
 	}})->{uri};
 }
 
-=item B<wait_project_enabled> PROJECT_URI
+=item B<create_user> DOMAIN EMAIL LOGIN PASSWORD FIRST_NAME LAST_NAME PHONE COMPANY SSO_PROVIDER
 
-Wait until project identified by its uri is in enabled state,
-return its identifier.
+Create a user given its email, login, password, first name, surname, phone and optionally company,
+sso provider in domain.
 
-=cut
-
-sub wait_project_enabled
-{
-	my $self = shift;
-	my $project_uri = shift || die 'Project uri was not specified.';
-
-	my $state;
-	my $exported = $self->poll (
-		sub { $self->{agent}->get ($project_uri) },
-		sub { $_[0] and exists $_[0]->{project} and exists $_[0]->{project}{content} and exists $_[0]->{project}{content}{state} and
-			(($state = $_[0]->{project}{content}{state}) !~ /^(PREPARING|PREPARED|LOADING)$/)
-		}
-	) or die 'Timed out waiting for project preparation';
-	($state eq 'ENABLED') or die "Unable to enable project";
-}
-
-=item B<create_user> LOGIN PASSWORD FIRST_NAME LAST_NAME PHONE COMPANY
-
-Create a user given its login, password, first name, surname, phone and optionally company,
-return his identifier.
+Returns user identifier (URI).
 
 =cut
 
 sub create_user
 {
 	my $self = shift;
-	my $login = shift;
+	my $domain_uri = shift || die "No domain specified";
+	my $email = shift || die "Email must be specified";
+	my $login = shift || $email;
 	my $passwd = shift;
 	my $firstname = shift;
 	my $lastname = shift;
 	my $phone = shift;
 	my $company = shift || '';
+	my $sso_provider = shift;
 
-	return $self->{agent}->post ('/gdc/account/domains/default/users', { #TODO links does not exists
+	return $self->{agent}->post ($domain_uri."/users", { #TODO links does not exists in REST API
 		accountSetting => {
 			login => $login,
+			email => $email,
 			password => $passwd,
 			verifyPassword => $passwd,
 			firstName => $firstname,
 			lastName => $lastname,
 			phoneNumber => $phone,
-			companyName => $company
+			companyName => $company,
+			($sso_provider ? (ssoProvider => $sso_provider) : ()),
 	}})->{uri};
 }
 
-=item B<get_roles>
+=item B<get_roles> PROJECT
 
-Gets project roles. Project is identified by its id.
-return array of project roles.
+Gets project roles.
+
+Return array of project roles.
 
 =cut
 
@@ -421,145 +417,6 @@ sub get_roles
 	return $self->{agent}->get (
 		$self->get_uri (new URI($project), 'roles'))->{projectRoles}{roles};
 }
-
-=item B<get_roles_by_id>
-
-Gets project roles. Project is identified by its id.
-return hash map role id => role uri.
-
-=cut
-
-sub get_roles_by_id
-{
-	my $self = shift;
-	my $project = shift;
-	my $rolesUris = $self->get_roles ($project);
-
-	my %roles;
-
-	foreach my $roleUri (@$rolesUris) {
-		my $role = $self->{agent}-> get ($roleUri);
-		my $roleId = $role->{projectRole}{meta}{identifier};
-		$roles{$roleId} = $roleUri;
-	}
-	return %roles;
-}
-
-=item B<assign_user> USER PROJECT ROLE
-
-Assign user to project.
-return his identifier.
-
-=cut
-
-sub assign_user
-{
-	my $self = shift;
-	my $user = shift;
-	my $project = shift;
-	my $role = shift;
-
-	my @userRoles = ($role);
-
-	return $self->{agent}->post ($self->get_uri (new URI($project),'users'), {
-		user => {
-			content => {
-				status => "ENABLED",
-				userRoles => \@userRoles
-			},
-			links => {
-				self => $user
-			}
-		}
-	});
-}
-
-=item B<schedule> PROJECT_URI CRON PARAMS HIDDEN_PARAMS
-
-Create a schedule given its project, type, cron expression and optionally
-parameters and hidden parameters, return created schedule object.
-
-=cut
-
-sub schedule {
-	my $self = shift;
-	my $project_uri = shift;
-	my $type = shift;
-	my $cron = shift;
-	my $params = shift || { };
-	my $hidden_params = shift || { };
-
-	return $self->{agent}->post ($project_uri.'/schedules', {schedule => { #TODO no link to schedules does not exists
-		type => $type,
-		params => $params,
-		hiddenParams => $hidden_params,
-		cron => $cron
-	}});
-}
-
-=item B<schedule_msetl_graph> PROJECT_URI TRANSFORMATION_ID GRAPH_NAME CRON PARAMS HIDDEN_PARAMS
-
-Create a MSETL schedule given its project, clover transformation id,
-clover graph to schedule, cron expression and optionally
-parameters and hidden parameters, return created schedule object.
-
-=cut
-
-sub schedule_msetl_graph {
-	my $self = shift;
-	my $project_uri = shift;
-	my $trans_id = shift;
-	my $graph = shift;
-	my $cron = shift;
-	my $params = shift || { };
-	my $hidden_params = shift || { };
-
-	my $type = "MSETL";
-
-	$params->{"TRANSFORMATION_ID"} = $trans_id;
-	$params->{"CLOVER_GRAPH"} = $graph;
-
-	return $self->schedule (
-		$project_uri, $type, $cron, $params, $hidden_params);
-}
-
-=item B<create_clover_transformation> PROJECT_URI TEMPLATE TRANSFORMATION_ID NAME
-
-Create a clover transformation given its project uri, template, clover
-transformation id in template and optionaly name, return created transformation
-object.
-
-=cut
-
-sub create_clover_transformation
-{
-	my $self = shift;
-	my $projectUri = shift;
-	my $template = shift;
-	my $transformation = shift;
-	my $name = shift || $transformation;
-
-	my $file = $transformation.'.zip';
-	my $path = '/uploads/'.$file;
-
-	# download clover transformation zip file from project template
-	my $content = $self->{agent}->get ($template.'/'.$file);
-
-	# upload clover transformation zip file
-	my $uploads = new URI ($self->get_uri ('uploads'));
-	$uploads->path_segments ($uploads->path_segments, $file);
-	$self->{agent}->request (new HTTP::Request (PUT => $uploads,
-		['Content-Type' => 'application/zip'], $content->{raw}));
-
-	# create transformation
-	return $self->{agent}->post ($projectUri."/etl/clover/transformations", { #TODO links does not exists
-		cloverTransformation => {
-			name => $name,
-			path => $path
-		}
-	});
-}
-
 =item B<reports> PROJECT
 
 Return array of links to repoort resources on metadata server.
@@ -590,8 +447,8 @@ sub compute_report
 
 	return $self->{agent}->post (
 		$self->get_uri (qw/xtab xtab-executor3/),
-		{ report_req => { report => $report }}
-	)->{reportResult2}{meta}{uri};
+		{ report_req => { report => ''.$report }}
+	);
 }
 
 =item B<export_report> REPORT FORMAT
@@ -611,7 +468,7 @@ sub export_report
 	my $result = $self->{agent}->post (
 		$self->get_uri (qw/report-exporter exporter-executor/),
 		{ result_req => { format => $format,
-			report => $self->compute_report ($report) }}
+			result => $self->compute_report ($report) }}
 	);
 
 	# This is for new release, where location is finally set correctly;
@@ -627,11 +484,15 @@ sub export_report
 	$exported = $self->{agent}->get ($exported->{uri}) if exists $exported->{uri};
 
 	# Gotten the correctly coded result?
-	return $exported->{raw} if $exported->{type} eq {
+	my $wanted = $format;
+	my %compat = (
 		png => 'image/png',
 		pdf => 'application/pdf',
 		xls => 'application/vnd.ms-excel',
-	}->{$format};
+		csv => 'text/csv',
+	);
+	$wanted = $compat{$wanted} if exists $compat{$wanted};
+	return $exported->{raw} if $exported->{type} =~ /^$wanted/;
 
 	die 'Wrong type of content returned';
 }
@@ -884,9 +745,9 @@ L<WWW::GoodData::Agent> -- GoodData API-aware user agent
 
 =head1 COPYRIGHT
 
-Copyright 2011, 2012 Lubomir Rintel
+Copyright 2011, 2012, 2013 Lubomir Rintel
 
-Copyright 2012 Adam Stulpa, Jan Orel, Tomas Janousek
+Copyright 2012, 2013 Adam Stulpa, Jan Orel, Tomas Janousek
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
